#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, sys, pathlib, argparse, string
import obt._globals
import obt.docker
from obt import dep, path
from obt.deco import Deco
deco = Deco()

def print_item(key,val):
 dstr = deco.inf(dockerid)
 kstr = deco.key(key)
 vstr = deco.val(val)
 print("%s.%s = %s"%(dstr,kstr,vstr))

###############################################################################

parser = argparse.ArgumentParser(description='obt.build docker launcher')
parser.add_argument('dockermodulename', metavar='D', type=str, help='a docker module to launch')

args, unknownargs = parser.parse_known_args()

_args = vars(args)

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

dockermodulename = _args["dockermodulename"]

obt._globals.setOption("dockermodulename",dockermodulename)

####################################
# invoke on docker module
####################################

dockermodule = obt.docker.descriptor(dockermodulename)
dockermodule.test()
sys.exit(0)


from obt import dep, path, command, docker, wget, pathtools
from obt.deco import Deco
import obt.module
import time, re, socket, os, sys
from pathlib import Path
deco = obt.deco.Deco()

this_dir = path.directoryOfInvokingModule()

makedotcfg = this_dir/"Makefile.cfg"

###############################################################################

class dockerinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self.type = docker.Type.SINGLE
      self._name = "ps1dev"
      self.imagename = "obt-ps1dev:latest"
    ###############################################
    # build the docker image
    ###############################################
    def build(self,build_args):
      os.chdir(this_dir)
      command.run([
        "docker", "build", ".", "-t", self.imagename ])
    ###############################################
    # kill active docker container
    ###############################################
    def kill(self):
      pass
    ###############################################
    # launch docker container
    #  print out connection info
    ###############################################
    def launch(self,launch_args,environment=None,mounts=None):
        command.run([
            "docker", "run",
            "-it",
            "--mount","type=bind,source=%s,target=/home/ps1dev/Makefile.cfg"%makedotcfg,
            self.imagename,
            "/bin/bash"
        ])
    ###############################################
    # launch docker container
    #  print out connection info
    ###############################################
    def test(self):
        os.chdir(this_dir)
        testprogdir = this_dir/"testprograms"
        builddir = path.builds()/"ps1dev-test1"
        pathtools.ensureDirectoryExists(builddir)
        command.run([
            "docker", "run",
            "--mount","type=bind,source=%s,target=/home/ps1dev/testprograms"%testprogdir,
            "--mount","type=bind,source=%s,target=/home/ps1dev/.build-out"%builddir,
            "--mount","type=bind,source=%s,target=/home/ps1dev/Makefile.cfg"%makedotcfg,
            #"-v","testprogram/Makefile:/home/ps1dev/testprogram/Makefile:rw",
            self.imagename,
            "/bin/bash", "-c", 
            #"find"
            "cd ~/testprograms/test1; make"
            #"cd ~/examples/mandel; make"
        ])
        print("builddir<%s>"%builddir)
        command.run(["ls","-l",builddir])
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "image_name": self.imagename,
      }
    ###############################################


from yarl import URL
from obt import dep, path

###############################################################################

class gnutar(dep.StdProvider):
  name = "gnutar"
  def __init__(self):
    super().__init__(gnutar.name)
    self._version = "1.34"
    baseurl = URL("ftp.gnu.org/gnu/tar")
    basename = "tar-%s"%self._version
    filename = "%s.tar.xz" % basename
    self.setSourceRoot(path.builds()/gnutar.name/basename)
    self._fetcher._url = baseurl/filename
    self._fetcher._fname = filename
    self._fetcher._arctype = "tgz"
    self._fetcher._md5 = "9a08d29a9ac4727130b5708347c0f5cf"
    self._builder = self.createBuilder(dep.AutoConfBuilder)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.WgetFetcher(gnutar.name)
  ########################################################################



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path

###############################################################################

class osgeolaszip(dep.StdProvider):
  name = "osgeolaszip"
  def __init__(self):
    super().__init__(osgeolaszip.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=osgeolaszip.name,
                             repospec="LASzip/LASzip",
                             revision="v2.2.0", # 2.2.0 is last version liblas is compatibile with!
                             recursive=False)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"liblaszip.so").exists()



#!/usr/bin/env python3
 
from obt import command, tmux

##########################################################

session = tmux.Session("OBT_SSL_TEST",orientation="horizontal")
session.command(["_obt.test.docker.ssl.server.py"])
session.command(["_obt.test.docker.ssl.client.py"])
session.execute()




#!/usr/bin/env python3

"""
Simple SSL connectivity test
Just verifies the SSL proxy is accepting connections on port 8443
"""

import ssl
import socket
import sys

def test_ssl_connectivity(host="localhost", port=8443):
    """Test that SSL proxy is accepting connections"""
    print(f"🔌 Testing SSL connectivity to {host}:{port}")
    
    try:
        # Create SSL context (ignore cert verification for demo)
        ssl_context = ssl.create_default_context()
        ssl_context.check_hostname = False
        ssl_context.verify_mode = ssl.CERT_NONE
        
        # Create and connect socket
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        ssl_sock = ssl_context.wrap_socket(sock, server_hostname=host)
        
        print(f"  Connecting...")
        ssl_sock.connect((host, port))
        print(f"  ✅ SSL connection established")
        print(f"  🔐 Cipher: {ssl_sock.cipher()}")
        print(f"  📡 SSL proxy is forwarding to ZMQ server")
        
        ssl_sock.close()
        return True
        
    except ConnectionRefusedError:
        print(f"  ❌ Connection refused - is the SSL proxy running?")
        return False
    except socket.timeout:
        print(f"  ❌ Connection timeout")
        return False
    except ssl.SSLError as e:
        print(f"  ❌ SSL error: {e}")
        return False
    except Exception as e:
        print(f"  ❌ Error: {e}")
        return False

def main():
    print("🧪 ZMQ SSL PROXY CONNECTIVITY TEST")
    print("=" * 50)
    
    if len(sys.argv) > 1:
        host = sys.argv[1]
    else:
        host = "localhost"
    
    # Run the test
    success = test_ssl_connectivity(host)
    
    print("=" * 50)
    if success:
        print("🎉 TEST PASSED: SSL proxy is accepting connections!")
        print("   The SSL termination proxy is working correctly")
        print("   ZMQ server communication requires proper ZMQ client")
        sys.exit(0)
    else:
        print("💥 TEST FAILED: SSL proxy is not reachable")
        print("   Check docker compose logs for details:")
        print("   docker compose logs ssl-proxy")
        print("   docker compose logs zmq-server")
        sys.exit(1)

if __name__ == "__main__":
    main() 


###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

VERSION = "v1.10.0"

import os, tarfile
from yarl import URL
from obt import dep, host, path, git, pathtools, patch
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command
import obt.host
import fileinput

deco = Deco()

###############################################################################

class lz4(dep.StdProvider):
  name = "lz4"
  def __init__(self): ############################################
    super().__init__(lz4.name)
    self._builder = dep.CustomBuilder(lz4.name)
    bdir = self.source_root

    cmdlist = ["make","-j",host.NumCores]

    if obt.host.IsOsx:
        cmdlist += ["MACOSX_DEPLOYMENT_TARGET=10.15"]

    clean_cmd = Command(cmdlist+["clean"],working_dir=bdir)
    install_cmd = Command(cmdlist+["install"],working_dir=bdir)

    self._builder._cleanbuildcommands += [clean_cmd,install_cmd]
    self._builder._incrbuildcommands = [install_cmd]
    self._builder._builddir = bdir

  ########

  def __str__(self):
    return "lz4 (lz4.org-source-%s)" % VERSION

  ########################################################################
  @property
  def _fetcher(self):
    makefile_items = dict()
    makefile_items["export PREFIX= /usr/local"]="export PREFIX=%s"%path.prefix()
    patch_dict = { self.source_root/"Makefile": makefile_items }
    return dep.GithubFetcher(name=lz4.name,
                             repospec="lz4/lz4",
                             revision=VERSION,
                             recursive=False,
                             patchdict=patch_dict)

  ########

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"lz4-1.10.0").exists()



from obt import dep, path, command, docker, wget, pathtools, host, target
from obt.deco import Deco
import obt.module
import time, re, socket, os, sys
from pathlib import Path
deco = obt.deco.Deco()

this_path = os.path.realpath(__file__)
this_dir = Path(os.path.dirname(this_path))

###############################################################################

class subspaceinfo:
    ###############################################
    def __init__(self):
      super().__init__()
      self._name = "host"
      is_linux = host.IsLinux 
      is_arm = host.IsAARCH64
      if is_linux:
        if is_arm:
          self._target = target.descriptor("aarch64","linux")
        else:
          self._target = target.descriptor("x86_64","linux")
      else:
        if is_arm:
          self._target = target.descriptor("aarch64","macos")
        else:
          self._target = target.descriptor("x86_64","macos")
    ###############################################
    # build the docker image
    ###############################################
    def build(self,build_args):
      os.chdir(this_dir)
      print("build host")
    ###############################################
    # launch docker container
    #  print out connection info
    ###############################################
    def launch(self,launch_args):
      print("launch host")
    ###############################################
    # information dictionary
    ###############################################
    def info(self):
      return {
        "name": "host",
        "prefix": path.stage(),
      }
    ###############################################




"""Bleached Markdown functionality.

This is for user-generated stuff, like comments.
"""

import bleach
import commonmark as CommonMark 

from . import shortcodes

ALLOWED_TAGS = [
    'a',
    'abbr',
    'acronym',
    'b', 'strong',
    'i', 'em',
    'del', 'kbd',
    'dl', 'dt', 'dd',
    'blockquote',
    'code', 'pre',
    'li', 'ol', 'ul',
    'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
    'p', 'br', 'hr',
    'sup', 'sub', 'strike',
    'img',
    'iframe',
    'video',
]

ALLOWED_ATTRIBUTES = {
    'a': ['href', 'title', 'target'],
    'abbr': ['title'],
    'acronym': ['title'],
    'img': ['src', 'alt', 'width', 'height', 'title'],
    'iframe': ['src', 'width', 'height', 'frameborder', 'allowfullscreen'],
    'video': ['autoplay', 'controls', 'loop', 'muted', 'src'],
    '*': ['style'],
}

ALLOWED_STYLES = [
    'color', 'font-weight', 'background-color',
]


def markdown(s: str) -> str:
    commented_shortcodes = shortcodes.comment_shortcodes(s)
    tainted_html = CommonMark.commonmark(commented_shortcodes)

    # Create a Cleaner that supports parsing of bare links (see filters).
    cleaner = bleach.Cleaner(tags=ALLOWED_TAGS,
                             attributes=ALLOWED_ATTRIBUTES,
                             styles=ALLOWED_STYLES,
                             strip_comments=False,
                             filters=[bleach.linkifier.LinkifyFilter])

    safe_html = cleaner.clean(tainted_html)
    return safe_html


def cache_field_name(field_name: str) -> str:
    """Return the field name containing the cached HTML.

    See ValidateCustomFields._normalize_coerce_markdown().
    """
    return f'_{field_name}_html'



#!/usr/bin/env python3

"""
ZMQ Ping Test through SSL Proxy
Tests the complete chain: TCP Client → SSL → stunnel → ZMQ Server
Uses raw TCP instead of ZMQ protocol to work through stunnel
"""

import socket
import ssl
import json
import sys
import time

def test_zmq_ping(host="localhost", port=8443):
    """Test ZMQ ping through SSL proxy using raw TCP"""
    print(f"🔌 Testing ZMQ ping through SSL proxy at {host}:{port}", flush=True)
    
    try:
        # Create SSL context (ignore cert verification for demo)
        ssl_context = ssl.create_default_context()
        ssl_context.check_hostname = False
        ssl_context.verify_mode = ssl.CERT_NONE
        
        # Create TCP socket and wrap with SSL
        raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        raw_socket.settimeout(5.0)  # 5 second timeout
        
        ssl_socket = ssl_context.wrap_socket(raw_socket, server_hostname=host)
        
        endpoint = f"{host}:{port}"
        print(f"  Connecting to SSL proxy: {endpoint}", flush=True)
        ssl_socket.connect((host, port))
        
        print(f"  ✅ SSL connection established", flush=True)
        print(f"  🔐 Cipher: {ssl_socket.cipher()}", flush=True)
        
        # Send ping command as raw JSON (no ZMQ framing)
        ping_request = {"command": "ping"}
        request_json = json.dumps(ping_request)
        print(f"  📤 Sending ping: {request_json}", flush=True)
        
        # Send as raw bytes
        ssl_socket.send(request_json.encode('utf-8'))
        print(f"  ⏳ Waiting for response (5 sec timeout)...", flush=True)
        
        # Receive response
        response_data = ssl_socket.recv(4096)
        response_json = response_data.decode('utf-8')
        print(f"  📥 Received: {response_json}", flush=True)
        
        # Parse and validate response
        try:
            response = json.loads(response_json)
            if response.get('status') == 'success' and response.get('message') == 'pong':
                print(f"  ✅ ZMQ ping successful!", flush=True)
                print(f"  🕐 Server timestamp: {response.get('timestamp')}", flush=True)
                ssl_socket.close()
                return True
            else:
                print(f"  ❌ Unexpected response: {response}", flush=True)
                ssl_socket.close()
                return False
        except json.JSONDecodeError as e:
            print(f"  ❌ Invalid JSON response: {e}", flush=True)
            ssl_socket.close()
            return False
            
    except socket.timeout:
        print(f"  ❌ Timeout - no response from ZMQ server", flush=True)
        try:
            ssl_socket.close()
        except:
            pass
        return False
    except ConnectionRefusedError:
        print(f"  ❌ Connection refused - is SSL proxy running?", flush=True)
        return False
    except ssl.SSLError as e:
        print(f"  ❌ SSL Error: {e}", flush=True)
        return False
    except Exception as e:
        print(f"  ❌ Error: {e}", flush=True)
        try:
            ssl_socket.close()
        except:
            pass
        return False

def main():
    print("🧪 ZMQ PING TEST (via SSL Proxy)", flush=True)
    print("=" * 50, flush=True)
    print("Testing: TCP Client → SSL → stunnel → ZMQ Server", flush=True)
    print("Using raw TCP instead of ZMQ protocol for stunnel compatibility", flush=True)
    print("", flush=True)
    sys.stdout.flush()
    
    if len(sys.argv) > 1:
        host = sys.argv[1]
    else:
        host = "localhost"
    
    # Run the test
    success = test_zmq_ping(host)
    
    print("=" * 50, flush=True)
    if success:
        print("🎉 PING TEST PASSED!", flush=True)
        print("   ✅ SSL proxy is working", flush=True)
        print("   ✅ stunnel is forwarding correctly", flush=True)
        print("   ✅ ZMQ server is responding", flush=True)
        print("   🔐 End-to-end SSL protection is active", flush=True)
        print("   📡 Raw TCP over SSL works through stunnel", flush=True)
        sys.exit(0)
    else:
        print("💥 PING TEST FAILED!", flush=True)
        print("   Debug steps:", flush=True)
        print("   1. Is SSL proxy running? (docker compose ps)", flush=True)
        print("   2. Check SSL proxy logs: (docker compose logs ssl-proxy)", flush=True)
        print("   3. Check ZMQ server logs: (docker compose logs zmq-server)", flush=True)
        print("   4. Try connectivity test: (make test)", flush=True)
        sys.exit(1)

if __name__ == "__main__":
    main() 


import os, tarfile
from obt import dep, host, path, make, pathtools, log
from obt.deco import Deco
from obt.wget import wget
from obt import command

VER = "0.29.2"
NAME = "pkgconfig"

class _pkgconfig_from_source(dep.StdProvider):

  def __init__(self): ############################################
    super().__init__(NAME,NAME)
    self._builder = self.createBuilder(dep.CustomBuilder)

    environ = dict()
    if host.IsDarwin and host.IsX86_64:
      environ["CC"] = "gcc-13"
      environ["CXX"] = "g++-13"

    self.scope = dep.ProviderScope.INIT
    cmd_factory_sdir = command.factory(environ={"NOCONFIGURE":1},wdir=self.source_root)
    cmd_factory_bdir = command.factory(environ=environ,wdir=self.build_dest)

    autogen = cmd_factory_sdir.cmd("./autogen.sh")
    mkdir = cmd_factory_sdir.cmd("mkdir",".build")
    configure = cmd_factory_bdir.cmd("../configure","--prefix=%s"%path.prefix(),"--with-internal-glib")
    if host.IsLinux and host.IsAARCH64:
      configure.append_args(["--build=aarch64-linux-gnu"])

    self._builder._cleanbuildcommands = [autogen,mkdir,configure]
    self.VERSION = VER

    if host.IsLinux and host.IsX86_64:
      f2r = path.stage()/"bin"/"x86_64-unknown-linux-gnu-pkg-config"
      os.system( "rm -f %s" % f2r)
    elif host.IsLinux and host.IsAARCH64:
      f2r = path.stage()/"bin"/"x86_64-unknown-linux-gnu-pkg-config"
      os.system( "rm -f %s" % f2r)

    pathtools.ensureDirectoryExists(path.pkgconfigdir())

    self._builder._incrbuildcommands += [cmd_factory_bdir.cmd("make")]
    self._builder._installcommands  = [cmd_factory_bdir.cmd("make","install")]


  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=NAME,
                             repospec="tweakoz/pkg-config",
                             revision="obt-pkg-config-%s"%VER,
                             recursive=False)
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"pkg-config-guide.html").exists()
  ########################################################################
  def areRequiredBinaryFilesPresent(self):
    return (path.bin()/"pkg-config").exists()
###############################################################################
class _pkgconfig_from_homebrew(dep.HomebrewProvider):
  def __init__(self):
    super().__init__(NAME,NAME)
    self.VERSION = "homebrew"
###############################################################################
class pkgconfig(dep.switch(linux=_pkgconfig_from_source, \
                           macos=_pkgconfig_from_homebrew)):
  def __init__(self):
    super().__init__()
  def env_init(self):
    log.marker("registering pkgconfig SDK(%s)"%self.VERSION)



#!/usr/bin/env python3

from obt import  path, command, wget, pathtools, host
from yarl import URL

assert(host.IsLinux and host.IsX86_64)

plugin_dir = path.home()/".docker"/"cli-plugins"
dl_url = URL("https://github.com")/"docker"/"buildx"/"releases"/"download"/"v0.11.2"/"buildx-v0.11.2.linux-amd64"
out_file = plugin_dir/"docker-buildx"

pathtools.ensureDirectoryExists(plugin_dir)
command.run(["curl","-sSL",dl_url,"-o", out_file],do_log=True)
command.run(["chmod","+x", out_file],do_log=True)




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os
from obt import dep, path, template, env, command, conan, subspace, host, target

CONAN_FILE = """
from conan import ConanFile
from conan.tools.cmake import CMake, cmake_layout, CMakeToolchain
from conan.tools.files import get
import os

class LexerTL14Conan(ConanFile):
    name = "lexertl14"
    version = "tweakoz-obt"
    license = "<License here>"
    author = "<Your Name> <Your Email>"
    url = "https://github.com/tweakoz/lexertl14"
    description = "<Description of lexertl14 here>"
    topics = ("<Put some tag here>", "<Put another tag here>")
    settings = "os", "compiler", "build_type", "arch"

    def generate(self):
        tc = CMakeToolchain(self)
        tc.generate()

    def layout(self):
        cmake_layout(self)

    def source(self):
        get(self, "https://github.com/tweakoz/lexertl14/archive/refs/heads/toz-oct16.zip", destination=self.source_folder, strip_root=True)

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()

    def package_info(self):
        self.cpp_info.libs = ["lexertl14"]
"""

###############################################################################

class lexertl14(dep.StdProvider):
  name = "lexertl14"
  def __init__(self):
    super().__init__(lexertl14.name,subspace_vif=2)
    self.scope = dep.ProviderScope.SUBSPACE    
    self.declareDep("cmake")
    tgt = target.current()
    ############################################
    if tgt != None and tgt.os=="ios":
    ############################################
      self._builder = self.createBuilder(dep.ConanBuilder)
      lexertl14_dir = path.conan_prefix() / "lexertl14"
      lexertl14_dir.mkdir(parents=True, exist_ok=True)  # Ensure the directory exists
      # write conanfile.py
      lrtl_output = lexertl14_dir / "conanfile.py"
      with open(lrtl_output, "w") as f:
          f.write(self._conanfile)  
      #
      self._builder._working_dir = lexertl14_dir
      os.chdir(str(lexertl14_dir))
      #conan export . lexertl14/tweakoz-obt@user/channel
      the_environ = os.environ.copy()
      the_environ.update(conan.environment())
      self._builder._environment = the_environ
      self._builder._cmdlist = \
        ["conan","export",".",
         "--user=user",
         "--channel=channel"]
      self._builder._cmdlist2 = \
        ["conan", "create", ".",
         f"--profile:host={path.subspace_dir()}/ios.host.profile",
         f"--profile:build={path.subspace_dir()}/ios.build.profile",
         "--user=user",
         "--channel=channel",
         "--build=missing"]
    ############################################
    else:
    ############################################
      self._builder = self.createBuilder(dep.CMakeBuilder)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=lexertl14.name,
                             repospec="tweakoz/lexertl14",
                             revision="toz-oct16",
                             recursive=False)

  #######################################################################
  @property
  def _conanfile(self):
    return CONAN_FILE
  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.subspace_includes()/"lexertl"/"char_traits.hpp").exists()



import obt.xcode 
import obt.command 
import obt.env
import obt.path 
import os, re, sys
import subprocess

class _iossdk_private:

  _instance = None

  def __new__(cls):
    if cls._instance is None:
      cls._instance = super().__new__(cls)
      cls._instance._initialize()
    return cls._instance

  def _initialize(self):

    self._xcodesdkstr = obt.command.capture([
      "xcodebuild",
      "-version",
      "-sdk"])

    lines = self._xcodesdkstr.splitlines()

    class SDKVER:
      def __init__(self, match):
        self.number = a.group(1)
        self.name = a.group(0)
        self.major,self.minor = self.number.split(".")
        self.path = obt.command.capture([
          "xcodebuild",
          "-version",
          "-sdk", self.name,
          "Path"],do_log=False).splitlines()

      def __repr__(self):
        return "%s : %s " % (self.name, self.path)

    self.versions = dict()    

    for line in lines:
      regex = re.compile(r"iphoneos(\d+.\d+)")    
      a = regex.search(line)
      if a is not None:
        sdk = SDKVER(a)
        self.versions[sdk.number] = sdk
      
    #print(self.versions)  

    sorted = list(self.versions.keys())
    sorted.sort()
    
    highest = self.versions[sorted[-1]]
    
    self._iossdkstr = highest.path
    self._iossdkver = highest.number

    print(f"Using SDK: {self._iossdkver} {self._iossdkstr}")
    
    try:
      # Use xcrun to find the path to clang for the current SDK
      clang_path = subprocess.check_output([
        "xcrun",
        "-find",
        "clang"
      ], text=True).strip()
      clangpp_path = subprocess.check_output([
        "xcrun",
        "-find",
        "clang++"
      ], text=True).strip()

      # Set environment variable for the compiler path
      self._clang_path = clang_path
      self._clangpp_path = clangpp_path

    except subprocess.CalledProcessError as e:
      print(f"Error executing command: {e}")
    except Exception as e:
      print(f"An unexpected error occurred: {e}")

    #assert(False)


class sdkinfo:
  #################################################
  # env_init invoked once at container startup time
  #################################################

  #############################################
  def __init__(self):
    self.identifier = "aarch64-ios"
    self.architecture = "aarch64"
    self.os = "ios"
    self.c_compiler = "clang"
    self.cxx_compiler = "clang++"
    self.supports_host = ["x86_64-macos","aarch64-macos"]
  #############################################
  @property
  def sdkdir(self):
    return obt.path.Path(os.environ["OBT_IOS_SDK_DIR"])
  @property
  def _sdkdir(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._iossdkstr
  @property
  def _sdkver(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._iossdkver
  @property
  def _clang_path(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._clang_path
  @property
  def _clangpp_path(self):
    IOS_SDK = _iossdk_private()
    return IOS_SDK._clang_path
  #############################################
  @property
  def _environment(self):
    IOS_SDK = _iossdk_private()
    return {
      "IOS_SDK_DIR": self._sdkdir,
      "IOS_SDK_VER": self._sdkver,
      "IOS_CLANG_PATH": self._clang_path,
      "IOS_CLANGPP_PATH": self._clangpp_path,
      #"CMAKE_TOOLCHAIN_FILE": prefix/"ios.toolchain.cmake",
      #"OBT_SUBSPACE_PROMPT": self._gen_sysprompt(),
      "OBT_TARGET": "aarch64-ios",
      "IOS_PREFIX": obt.path.subspace_root()/"ios",
      #"OBT_SUBSPACE": "ios",
    }
  #############################################
  def install_app(self,bundle_id,app_bundle_dir):
    ##############################################
    # Check if the device is connected
    ##############################################
    device_info = subprocess.check_output(["idevice_id", "-l"], universal_newlines=True).strip()
    if not device_info:
      print("No iOS device connected. Please connect an iOS device and try again.")
      sys.exit(1)
    ##############################################
    # Install the app on the connected device
    ##############################################
    print(f"Installing app {bundle_id} from {app_bundle_dir} to iphone")
    subprocess.check_output(["ideviceinstaller", "-U", bundle_id])
    subprocess.check_output(["ideviceinstaller", "-i", app_bundle_dir])
  #############################################
  def misc(self):
    return {
      "sdkdir": self.sdkdir,
      "_sdkdir": self._sdkdir,
      "_sdkver": self._sdkver,
      "_clang_path": self._clang_path,
      "_clangpp_path": self._clangpp_path
    }




from obt import dep, pip, path, dep
from obt.command import Command
###############################################################################

class pyside2(dep.Provider):

  def __init__(self): ############################################
    super().__init__("pyside2")
    build_dest = path.builds()/"pyside2"
    self.build_dest = build_dest
    self.python = self.declareDep("python")

  def build(self): ############################################################
      return pip.install(["pyside2"])==0

  def areRequiredSourceFilesPresent(self):
    return (self.python.site_packages_dir/"PySide2"/"__init__.py").exists()

  def areRequiredBinaryFilesPresent(self):
    return self.areRequiredSourceFilesPresent()
